<?php
/**
 * Append comment to discussion raw wiki file
 *
 * @author Pavel Vitis <pavel [dot] vitis [at] seznam [dot] cz>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_saveComment($dID, $name, $email, $comment){
  global $TEXT;
  global $ACT;
  
  // Get raw text filename from discussion ID
  $dFN = wikiFN($dID);
  if (@file_exists($dFN)){
  
    // check if unregistred user tries to comment flood
    if (blog_checktempblock()){
      msg('Sorry, spamming is not allowed.', 0);
      return 'wordblock';
    }
  
    $TEXT = blog_compileCommentMarkup($name, $email, $comment);
    
    if (checkwordblock()){
      msg('Sorry, spamming is not allowed.', 0);
      return 'wordblock';
    } else {
      $old = saveOldRevision($dID);
      
      if (blog_getConf('comment_recentfirst')){
         $content = io_readFile($dFN);
         $TEXT = "----\n\n".$TEXT."\n\n".$content;
         io_saveFile($dFN, $TEXT);
      } else {
         $TEXT = "\n\n".$TEXT."\n\n----";
         io_saveFile($dFN, $TEXT, true);
      }
      
      addLogEntry(@filemtime($dFN), $dID, '', false);
      // send notify mails
      notify($dID, 'admin', $old, '', false);
      notify($dID, 'subscribers', $old, '', false);
    }
    $ACT = 'show';
  }
}

/**
 * Prepare comment DokuWiki markup
 *
 * @author Pavel Vitis <pavel [dot] vitis [at] seznam [dot] cz>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_compileCommentMarkup($name, $email, $comment){
  global $conf;
  global $lang;

  // use gravatar?
  if (blog_getConf('comment_usegravatar') && trim($email, " \t\n") != ''){
    $markup = '{{gravatar>'.$email.' |'.$name.'}} ';
  } else {
    $markup = '';
  }
  if (trim($name, " \t\n") == '') $name = 'anonymous';
  $markup .= '<sub>';
  if (blog_getConf('comment_linkemail') && trim($email, " \t\n") != ''){
    $markup .= '[['.$email.'|'.$name.']]';
  } else {
    $markup .= $name;
  }
  $markup .= ', '.date($conf['dformat']).':</sub>'."\n\n";
  $markup .= $comment;

  return $markup;
}

/**
 * Display existing comments if availaible
 *
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_discussionPage($dID){
  global $ID;
   
  $dFN = wikiFN($dID);
  if (file_exists($dFN)){
    echo p_cached_xhtml($dFN);
    // display an edit button for the discussion page if permitted
    if (auth_quickaclcheck($ID) >= blog_getConf('comment_authedit')
      && is_writable($dFN)){
      echo '<div class="secedit">';
      echo html_btn('edit', $dID, '', array('do' => 'edit'), 'post');
      echo '</div>';
    }
  }
}

/**
 * This displays the comment form
 *
 * @author Andreas Gohr <andi@splitbrain.org>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 * @author Dave Lawson <dlawson@masterytech.com>
 */
function blog_commentForm($dID){
  global $INFO;
  global $lang;
  global $conf;
  global $ID;

  // only if we are NOT in a comment page itself
  if (blog_isDiscussionID($ID)){
    $mainID = blog_stripDiscussionNS($ID);
    echo '<hr />';
    echo '<div class="level1">'.html_wikilink($mainID,$lang['img_back'].' '.$mainID).'</div>';
    return true;
  }

  // guest comments must be allowed or you need to be logged in
  // comments by unregistred users are disabled after a certain time defined in
  // $conf['allowguests'] to prevent spam
  if (($_SERVER['REMOTE_USER'] == '') && (!blog_getConf('comment_allowguests')
    || time() > @filemtime(wikiFN($ID))+blog_getConf('comment_allowguests')
    || blog_checkunderattack()))
    return '';

?>
<div class="centeralign">
  <form id="blog__comment_form" method="post" action="<?php echo script() ?>" accept-charset="<?php echo $lang['encoding']?>" onsubmit="return validate(this);">
  <div class="no">
    <input type="hidden" name="id" value="<?php echo $dID?>" />
    <input type="hidden" name="do" value="show" />
    <input type="hidden" name="blog" value="addcomment" />
    <?php
    // who wants to comment?
    if ($conf['useacl'] && $_SERVER['REMOTE_USER']){ // registered user
    ?>
      <input type="hidden" name="fullname" value="<?php echo $INFO['userinfo']['name']?>" />
      <input type="hidden" name="email" value="<?php echo $INFO['userinfo']['mail']?>" />
    <?php
    } else { // guest
    ?>
    <div class="comment_name">
      <label class="block" for="comment_name">
        <span><?php echo $lang['fullname'].':'?></span>
        <input type="text" class="edit" name="fullname" id="comment_name" size="40" tabindex="1" />
      </label>
    </div>
    <div class="comment_mail">
      <label class="block" for="comment_mail">
        <span><?php echo $lang['email'].':'?></span>
        <input type="text" class="edit" name="email" id="comment_mail" size="40" tabindex="2" />
      </label>
    </div>
    <?php
    }
    // coComment support - @todo: unobtrusive javascript
    ?>
    <div class="comment_text">
      <textarea class="edit" name="wikitext" cols="80" rows="10" id="comment_text"  tabindex="3"></textarea>
    </div>
    <input class="button" type="submit" name="submit" value="<?php echo $lang['btn_save']?>" accesskey="s" title="ALT+S" tabindex="4" />
  </div>
  </form>
  <script type="text/javascript">
    var blogTool               = "DokuWiki";
    var blogURL                = "<?php echo DOKU_URL ?>";
    var blogTitle              = "<?php echo $conf['title'] ?>";
    var postURL                = "<?php echo wl($ID, '', true) ?>";
    var postTitle              = "<?php echo tpl_pagetitle($ID) ?>";
    <?php if ($_SERVER['REMOTE_USER']){ ?>
    var commentAuthor          = "<?php echo $INFO['userinfo']['name'] ?>";
    <?php } else { ?>
    var commentAuthorFieldName = "fullname";
    <?php } ?>
    var commentAuthorLoggedIn  = <?php echo (($_SERVER['REMOTE_USER']) ? 'true' : 'false') ?>;
    var commentFormID          = "blog__comment_form";
    var commentTextFieldName   = "wikitext";
    var commentButtonName      = "submit";

    var cocomment_force        = false;
  </script>

  <script type="text/javascript" scr="http://www.cocomment.com/js/cocomment.js"></script>
</div>
<?php
}

/**
 * Prevents the flooding of our blog with lots of comments within a short time
 *
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_checktempblock(){
  global $conf;
  
  // we trust registered users
  if ($conf['useacl'] && $_SERVER['REMOTE_USER']) return false;
  
  // block all if site is under spam attack
  if (blog_checkunderattack()) return true;
  
  // no temporary blocking with savemode
  if ($conf['safemodehack']) return false;
  
  $ip           = clientIP();
  $tempblockdir = $conf['cachedir'].'/_tempblock';
  $blockfile    = $tempblockdir.'/'.$ip.'.block';
  $banfile      = $tempblockdir.'/'.$ip.'.ban';
    
  if (@file_exists($banfile)){
  
    // this ip is still banned
    if (@filemtime($banfile)+(60*60*24) > time()){
      @touch($banfile);
      return true;
      
    // release the ban for this ip
    } else {
      @unlink($banfile);
      return false;
    }
  }
      
  // this user has already commented in the last 5 minutes
  if (@file_exists($blockfile)){
    if (@filemtime($blockfile)+300 > time()){
      // ban this ip for 24 hours by creating the banfile
      io_saveFile($banfile, '');
      
      // clean up the directory
      if ($dh = @opendir($tempblockdir)){
        while (($file = readdir($dh)) !== false){
          if (@filemtime($tempblockdir.$file)+(60*60*24) < time())
            @unlink($tempblockdir.$file);
        }
        closedir($dh);
      }
      
      return true;
      
    // this blockfile has expired
    } else {
      return false;
    }
    
  // we don't know this ip yet
  } else {
    $lastcomment  = $tempblockdir.'/last_comment.txt';
    
    io_saveFile($blockfile, '');
    $num = 1;
    
    // another comment within less than 10 seconds?
    if (@file_exists($lastcomment) && @filemtime($blockfile)+10 > time()){
      $text = file($lastcomment);
      $num  = $text[0]+1;
      
      // third comment in 10 seconds: turn commenting off for 5 minutes
      if ($num > 2){
        if (!defined(UNDERATTACK))
          define(UNDERATTACK, $tempblockdir.'/under_attack.txt');
          
        io_saveFile(UNDERATTACK, '');
        io_saveFile($lastcomment, 0);
        return true;
      }
    }    
    io_saveFile($lastcomment, $num);
    return false;
  }
}

/**
 * Turn off commenting for five minutes if we are under a spam attack
 *
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_checkunderattack(){
  global $conf;
  if (!defined(UNDERATTACK))
    define(UNDERATTACK, $conf['cachedir'].'/_tempblock/under_attack.txt');
  
  if (!@file_exists(UNDERATTACK)) return false;
  if (@filemtime(UNDERATTACK)+300 < time()){
    @unlink(UNDERATTACK);
    return false;
  } else {
    return true;
  }
}

//Setup VIM: ex: et ts=2 enc=utf-8 :
