<?php 
/**
 * Output XML Feed
 *
 * @author Andreas Gohr <andi@splitbrain.org>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
require_once(DOKU_INC.'inc/feedcreator.class.php');
require_once(DOKU_PLUGIN.'blog/inc/recent.php');
global $conf;

session_write_close();
  
$num  = $_REQUEST['num'];
$ns   = $_REQUEST['ns'];
$tag  = $_REQUEST['tag'];
$type = $_REQUEST['type'];

if (!is_numeric($num)) $num = $conf['recent'];
if ($tag) $ns = $tag;
elseif (!isset($ns)) $ns = blog_getConf('blog_namespace');
if (!isset($type)) $type = $conf['rss_type'];
if ($conf['useslash']) $ns = strtr($ns,'/',':');

switch ($type){
  case 'rss':
     $type = 'RSS0.9';
     break;
  case 'rss2':
     $type = 'RSS2.0';
     break;
  case 'atom':
     $type = 'ATOM0.3';
     break;
  default:
     $type = 'RSS1.0';
}

// the feed is dynamic - we need a cache for each combo
// (but most people just use the default feed so it's still effective)
$cache = getCacheName($num.$type.$ns.$tag.$_SERVER['REMOTE_USER'], '.feed');

// check cacheage and deliver if nothing has changed since last
// time (with 5 minutes settletime)
$cmod = @filemtime($cache); // 0 if not exists
if($cmod && ($cmod+(5*60) >= @filemtime($conf['changelog']))){
  header('Content-Type: application/xml; charset=utf-8');
  print io_readFile($cache);
  exit;
}

// create new feed
$rss = new DokuWikiFeedCreator();
$rss->title = ucwords($conf['title'].(($ns) ? ' '.$ns : ''));
$rss->link  = DOKU_URL;
$rss->syndicationURL = DOKU_URL.'lib/plugins/blog/inc/feed.php';
$rss->cssStyleSheet  = DOKU_URL.'lib/styles/feed.css';

$image = new FeedImage();
$image->title = $conf['title'];
$image->url = DOKU_URL.'lib/tpl/'.$conf['template'].'/images/favicon.ico';
$image->link = DOKU_URL;
$rss->image = $image;

blog_feedItems($rss, $num, $ns, $tag);

$feed = $rss->createFeed($type,'utf-8');

// save cachefile
io_saveFile($cache, $feed);

// finally deliver
header('Content-Type: application/xml; charset=utf-8');
print $feed;
exit;

/* ---------- */

/**
 * Add recent changed to a feed object
 *
 * @author Andreas Gohr <andi@splitbrain.org>
 */
function blog_feedItems(&$rss, $num, $ns, $tag){
  global $conf;
  global $auth;

	$flags = RECENTS_SKIP_DELETED + RECENTS_SKIP_MINORS;
  
  // tag feed
  if ($tag){
    $data = blog_tagFeedData($tag);
    $recents = blog_recents(0, $num, '', $flags, 'tag', $data);
    
  // namespace feed
  } else {
    if ($ns == '*') $ns = '';
    else $flags += RECENTS_SKIP_SUBSPACES;
    $recents = blog_recents(0, $num, $ns, $flags);
  }

  //this can take some time if a lot of recaching has to be done
  @set_time_limit(90); // set max execution time

	foreach($recents as $recent){

    $id    = $recent['id'];
    $item  = new FeedItem();
    $xhtml = p_wiki_xhtml($id, '', false);
    
    // first heading as title
    $matches = array();
    if (preg_match('|<h(\d)><a.*?>(.*?)</a></h\d>|', $xhtml, $matches))
       $item->title    = trim($matches[2]);
    else $item->title  = $id;
    
    // first tag as category
    if (preg_match('|<div id="blog__tags">(.*?)</div>|', $xhtml, $matches)){
       $cat            = strip_tags($matches[2]);
       $matches        = explode(',' , $cat);
       $item->category = $matches[0];
    }
    
    $item->link        = wl($id, '', true);

    $item->description = blog_cleanDesc($xhtml);
    $item->date        = date('r',$recent['date']);
		$cat = getNS($recent['id']);
		if ($cat) $item->category = $cat;
    
    $user = null;
    $user = @$recent['user']; // the @ spares time repeating lookup
    $item->author = '';
    
    if ($user){
      $userInfo = $auth->getUserData($user);
      $item->author = $userInfo['name'];
      $item->authorEmail = obfuscate($userInfo['mail']);
    } else {
      $item->authorEmail = 'anonymous@'.$recent['ip'];
    }
    $rss->addItem($item);
  }
}

/**
 * Clean description for feed inclusion
 *
 * Convert the blog entry to raw text
 *
 * @author Andreas Gohr <andi@splitbrain.org>
 * @author Esther Brunner <esther@kaffeehaus.ch>
 */
function blog_cleanDesc($desc){
  $parts = preg_split('/<hr.*>/i', $desc);
  if (count($parts) > 1) $desc = $parts[0];
  $desc = preg_replace('/[\n\r\t]/',' ',$desc);
  $replace = array(
    '!<div class="toc">.*?(</div>\n</div>)!s' => '', // remove toc
    '!<h1>(.*?)</h1>!s'                       => '', // remove title
    '#<!-- SECTION \[(\d*-\d*)\] -->#e'       => '', // remove section edit buttons
    '!<div id="blog__tags">.*?(</div>)!s'     => '', // remove category tags
 // '!</(p|h\d)>!s'                           => "\n\n",  // end of paragraphs
 // '!<li.*>!s'                               => '* ', // list items
 // '!<(br.*|/li|/ul|/ol|/tr|/th|/table)>!s'  => "\n" // line breaks
  );
  $desc = preg_replace(array_keys($replace), array_values($replace), $desc);
  $desc = strip_tags($desc);
  $desc = preg_replace('/  /',' ',$desc);
  if (count($parts) > 1) $desc = $desc.'...';
  return $desc;
}

function blog_tagFeedData($tag){
  require_once (DOKU_INC.'inc/indexer.php');
  require_once (DOKU_INC.'inc/fulltext.php');
  
  $result = array();
  
  // quick lookup of the pagename
  $sw    = array(); // we don't use stopwords here
  $matches = idx_lookup(idx_tokenizer($tag, $sw));  // pagename may contain specials (_ or .)
  $docs  = array_keys(ft_resultCombine(array_values($matches)));
  $docs  = array_filter($docs,'isVisiblePage'); // discard hidden pages
  if (!count($docs)) return $result;

  // check instructions for matching links
  foreach ($docs as $match){
    $matchFN = wikiFN($match);
    $instructions = p_cached_instructions($matchFN, true);
    if (is_null($instructions)) continue;
          
    foreach ($instructions as $ins){
      if (($ins[0] == 'plugin') && ($ins[1][0] == 'blog_tag')){
        foreach ($ins[1][1] as $word){
          if (utf8_strtolower($tag) == utf8_strtolower($word)){
            //we have a match - finish
            $result[] = $match;
            break 2;
          }
        }
      }
    }
  }
  
  if (!count($result)) return $result;
    
  // check ACL permissions
  foreach (array_keys($result) as $idx){
    if (auth_quickaclcheck($result[$idx]) < AUTH_READ) unset($result[$idx]);
  }
  
  return $result;
}

//Setup VIM: ex: et ts=2 enc=utf-8 :
