<?php
/**
 * adapted version of getRecents()
 *
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @author  Esther Brunner <esther@kaffeehaus.ch>
 */
require_once(DOKU_PLUGIN.'blog/functions.php');
 
function blog_recents($first, $num, $ns='', $flags=0, $type='', $data=''){
  global $conf;
  $recent = array();
  $seen   = array();
  $count  = 0;
    
  if(!@is_readable($conf['changelog'])){
    msg($conf['changelog'].' is not readable', -1);
    return $recent;
  }
  if (($type) && (!$data)) return $recent;
      
  $fh  = fopen($conf['changelog'], 'r');
  $buf = '';
  $csz = 4096;                // chunksize
  fseek($fh, 0, SEEK_END);    // jump to the end
  $pos = ftell($fh);          // position pointer
  
  // now read backwards into buffer
  while($pos > 0){
    $pos -= $csz;             // seek to previous chunk...
    if ($pos < 0) $pos = 0;   // ...or rest of file
    fseek($fh, $pos);
    
    $buf = fread($fh, $csz).$buf;  // prepend to buffer
    
    $lines = explode("\n", $buf);  // split buffer into lines
    
    if ($pos > 0){
      $buf = array_shift($lines);  // first one may be still incomplete
    }
    
    $cnt = count($lines);
    if (!$cnt) continue;           // no lines yet
    
    // handle lines
    for ($i = $cnt-1; $i >= 0; $i--){
      $line = $lines[$i];
      if (empty($line)) continue;   // skip empty lines
      
      // split the line into parts
      list($dt, $ip, $id, $usr, $sum) = explode("\t", $line);
      
      // entry and exit clauses for monthly archive
      if ($type == 'month'){
        if ($dt > $data[1]) continue; // too new
        if ($dt < $data[0]){          // the rest is none of our business
          $pos = 0;                   // will break the while loop
          break;                      // will break the for loop
        }
      }
            
      if ($seen[$id]) continue;       // skip seen ones

      // entry clauses for user, ip and tag filtering
      switch ($type){
      case 'user':
        if (($usr != $data) && ($data != '@ALL')) continue 2;
        else break;
      case 'ip':
        if ($ip != $data) continue 2;
        else break;
      case 'tag':
        if (!in_array($id, $data)){
          $seen[$id] = 1;
          continue 2;
        }
      }
            
      if ($flags & RECENTS_SKIP_MINORS){
        if (isMinor($sum)) continue;  // skip minor changes
      }

      
      if (!$flags & RECENTS_SKIP_SUBSPACES){
        $id = blog_stripDiscussionNS($id); // no discussion pages, normal id
      }
      
      $seen[$id] = 1;                 // remember we already checked that id
      
      if (isHiddenPage($id)) continue; // skip excluded pages
      if ($flags & RECENTS_SKIP_SUBSPACES){
        if (getNS($id) != $ns) continue; // skip subspaces
      } else {                        // filter namespace
        if (($ns) && (strpos($id, $ns.':') !== 0)) continue;
      }

      if (auth_quickaclcheck($id) < AUTH_READ) continue; // check ACL
      if ($flags & RECENTS_SKIP_DELETED){
        if (!@file_exists(wikiFN($id))) continue; // skip deleted
      }
      
      if (--$first >= 0) continue;    // skip first entries
      
      $recent[] = array(
        'id'   => $id,
        'date' => $dt,
        'user' => $usr
      );
      $count++;
      
      // break while when we have enough entries
      if (($type != 'month') && ($count >= $num)){
        $pos = 0; // will break the while loop
        break;    // will break the for loop
      }
    }
  } // end of while
  
  fclose($fh);
  return $recent;  
}

//Setup VIM: ex: et ts=4 enc=utf-8 :