<?php
/**
 * Discussion Plugin
 *
 * Enables/disables discussion features based on config settings.
 *
 * @license GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author  Dave Lawson <dlawson@masterytech.com>
 * @author  Esther Brunner <esther@kaffeehaus.ch>
 */

if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
require_once(DOKU_PLUGIN.'blog/functions.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_discussion extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Dave Lawson',
      'email'  => 'dlawson@masterytech.com',
      'date'   => '2006-03-10',
      'name'   => 'Blog Plugin (discussion component)',
      'desc'   => 'Enables discussion features',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog:new',
    );
  }

  function getType(){ return 'substition'; }
  function getPType(){ return 'block'; }
  function getSort(){ return 230; }

  /**
   * Connect pattern to lexer
   */
  function connectTo($mode){
    if ($mode == 'base'){
      $this->Lexer->addSpecialPattern('~~DISCUSSION~~', $mode, 'plugin_blog_discussion');
      $this->Lexer->addSpecialPattern('~~NODISCUSSION~~', $mode, 'plugin_blog_discussion');
    }
  }

  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
    global $ID;
    global $ACT;

    $dID = blog_addDiscussionNS($ID);
    $dFN = wikiFN($dID);

    if ($match == '~~DISCUSSION~~' && !blog_isDiscussionID($ID)){
      // create or restore discussion page
      // but only if we are not on the main blog page
      if ((!file_exists($dFN))
        && (!defined('IS_BLOG_MAINPAGE'))
        && ($ACT == 'show')){
        // Current discussion page doesn't exist.
        $revs = getRevisions($dID);
        if (count($revs)) {
          // Restore previously delete discussion page.
          $rev = array_shift($revs);
          $txt = rawWiki($dID, $rev);
          $sum = $this->getLang('restored').$rev;
        } else {
          // Create new discussion page.
          $txt = ' ';
          $sum = $this->getLang('created');
        }
        saveWikiText($dID, $txt, $sum, true);
      }
      $state = 1;
    } elseif ($match == '~~NODISCUSSION~~'){
      // double-check before deleting the discussion page
      if (file_exists($dFN) && blog_isDiscussionID($dID)){
        saveWikiText($dID, '', $this->getLang('deleted'), true);
      }
      $state = 0;
    }

    return array($state);
  }

  /**
   * Create output
   */
  function render($mode, &$renderer, $data){
    if ($data[0]){
      $title = blog_getConf('discussion_title');
      $hid   = $renderer->_headerToLink($title, 'true');
      $renderer->toc[] = array(
        'hid'   => $hid,
        'title' => $title,
        'type'  => 'ul',
        'level' => 1
      );
    }

    return true;
  }

}

//Setup VIM: ex: et ts=4 enc=utf-8 :