<?php
/**
 * Include Plugin: displays a wiki page within another
 * Usage:
 * {{page>page}} for "page" in same namespace
 * {{page>:page}} for "page" in top namespace
 * {{page>namespace:page}} for "page" in namespace "namespace"
 * {{page>.namespace:page}} for "page" in subnamespace "namespace"
 * {{page>page#section}} for a section of "page"
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Esther Brunner <esther@kaffeehaus.ch>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_blog_page extends DokuWiki_Syntax_Plugin {

  /**
   * return some info
   */
  function getInfo(){
    return array(
      'author' => 'Esther Brunner',
      'email'  => 'esther@kaffeehaus.ch',
      'date'   => '2006-04-14',
      'name'   => 'Blog Plugin (page component)',
      'desc'   => 'Displays a wiki page (or a section thereof) within another',
      'url'    => 'http://wiki.splitbrain.org/plugin:blog',
    );
  }

  function getType(){ return 'substition'; }
  function getSort(){ return 303; }
  function getPType(){ return 'block'; }
  function connectTo($mode) { $this->Lexer->addSpecialPattern("{{page>.+?}}",$mode,'plugin_blog_page'); }
    
  /**
   * Handle the match
   */
  function handle($match, $state, $pos, &$handler){
      $match = substr($match, 7, -2);          // strip markup
      $match = preg_split('/\#/u', $match, 2); // split hash from filename

      return array($match[0], cleanID($match[1]));
  }    

  /**
   * Create output
   */
  function render($mode, &$renderer, $data) {
    global $ID;
    global $filechain;
    
    $id      = $this->_applyMacro($data[0]);
    resolve_pageid(getNS($ID), $id, $exists); // resolve shortcuts
    
    if($mode == 'xhtml'){
      require_once(DOKU_PLUGIN.'blog/inc/include.php');
      
      if (!isset($filechain)) $filechain[] = $ID;
      
      $file    = wikiFN($id);
      $section = $data[1];
      
      // don't allow to include the same page twice
      if (in_array($id, $filechain)) return false;
      $filechain[] = $id;
      
      // prevent caching to ensure the included page is always fresh
      $renderer->info['cache'] = FALSE;
      
      // check for permission
      $perm = auth_quickaclcheck($id);
      if ($perm < AUTH_READ) return true;
      
      // current section level
      preg_match_all('|<div class="level(\d)">|i', $renderer->doc, $matches, PREG_SET_ORDER);
      $n = count($matches)-1;
      if ($n > -1) $clevel = $matches[$n][1];
      else $clevel = 0;
      
      // get instructions and render them on the fly
      $instr   = p_cached_instructions($file, false);
      if ($section) $instr = incl_getSection($section, $instr);
      $instr   = incl_convertInstructions($instr, $id, $renderer, $clevel, $ok);
      $content = p_render('xhtml', $instr, $info);
      $content = incl_cleanXHTML($content, $tags);
                          
      // embed the included page
      if ($clevel && $ok) $renderer->doc .= '</div>';
      $renderer->doc .= '<div class="include">';
      $renderer->doc .= $content.incl_editButton($id, $file, $perm);
      $renderer->doc .= '</div>';
      if ($clevel && $ok) $renderer->doc .= '<div class="level'.$clevel.'">';
              
      return true;
      
    // for metadata renderer
    } elseif ($mode == 'metadata'){
      $renderer->meta['relation']['haspart'][$id] = $exists;
      // $meta = array('relation' => array('ispartof' => array($ID => true)));
      // p_set_metadata($id, $meta);

      return true;
    }
    return false;
  }
  
  function _applyMacro($id){
    global $INFO;
    
    $replace = array(
      '@USER@'  => cleanID($_SERVER['REMOTE_USER']),
      '@NAME@'  => cleanID($INFO['userinfo']['name']),
      '@YEAR@'  => date('Y'),
      '@MONTH@' => date('m'),
      '@DAY@'   => date('d'),
    );
    return str_replace(array_keys($replace), array_values($replace), $id);
  }
  
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :
